<?php

/**
 * E2pdf Convert Helper
 * 
 * @copyright  Copyright 2017 https://e2pdf.com
 * @license    GPL v2
 * @version    1
 * @link       https://e2pdf.com
 * @since      1.01.02
 */
if (!defined('ABSPATH')) {
    die('Access denied.');
}

class Helper_E2pdf_Convert {

    public function to_hex_color($hex = '') {
        $color = array(
            0x00, 0x00, 0x00
        );
        $hex = ltrim($hex, '#');
        if (strlen($hex) === 3) {
            $color = array(
                hexdec(substr($hex, 0, 1)),
                hexdec(substr($hex, 1, 1)),
                hexdec(substr($hex, 2, 1))
            );
        } elseif (strlen($hex) === 6) {
            $color = array(
                hexdec(substr($hex, 0, 2)),
                hexdec(substr($hex, 2, 2)),
                hexdec(substr($hex, 4, 2))
            );
        }
        return $color;
    }

    /**
     * Convert from bytes to Human View
     * 
     * @param int $size - Size in Bytes
     * 
     * @return string - Converted size
     */
    public function to_bytes($size) {
        $unit = array('B', 'KB', 'MB', 'GB', 'TB', 'PB');
        return @round($size / pow(1024, ($i = floor(log($size, 1024)))), 2) . '' . $unit[$i];
    }

    /*
      stritr - case insensitive version of strtr
      Author: Alexander Peev
      Posted in PHP.NET
     */

    public function stritr($haystack, $needle) {
        if (is_array($needle)) {
            $pos1 = 0;
            $result = $haystack;
            while (count($needle) > 0) {
                $positions = array();
                foreach ($needle as $from => $to) {
                    if (( $pos2 = stripos($result, $from, $pos1) ) === FALSE) {
                        unset($needle[$from]);
                    } else {
                        $positions[$from] = $pos2;
                    }
                }
                if (count($needle) <= 0) {
                    break;
                }

                $winner = min($positions);
                $key = array_search($winner, $positions);
                $result = ( substr($result, 0, $winner) . $needle[$key] . substr($result, ( $winner + strlen($key))) );
                $pos1 = ( $winner + strlen($needle[$key]) );
            }
            return $result;
        } else {
            return $haystack;
        }
    }

    public function to_shortcode_array($value, $fields = array()) {
        $list = explode(',', $value);
        $data = array();
        foreach ($list as $item) {
            $data[] = $this->parse_fields($item);
        }
        return $data;
    }

    private function parse_fields($value) {
        $data = array();
        $fields = explode("|", $value);
        foreach ($fields as $field) {
            $field_data = explode(":", $field);
            $field_key = $field_data['0'];
            unset($field_data['0']);
            $field_value = implode(":", $field_data);
            $data = array_merge_recursive($data, $this->parse_field($field_key, $field_value));
        }
        return $data;
    }

    private function parse_field($field_key, $field_value) {
        $keys = array_reverse(explode('_', $field_key));
        foreach ($keys as $key) {
            $field_value = [$key => $field_value];
        }
        return $field_value;
    }

}
